# Copyright (c) 2002-2007, RedIRIS. All Rights Reserved.
#
# You may distribute under the terms of the GNU General Public License,
# as specified in the LICENSE file that was shipped with this distribution.

package PAPI::RedirectGPoA;

use URI::Escape;
use PAPI::ApachePoA;
use PAPI::Cookie_Handler;
use APR::URI ();
use Data::UUID;
use Apache2::URI ();

sub handler
{
	my $r=shift;
	my $poa=shift;
	if (!defined $poa) {
		$poa=new PAPI::ApachePoA;
		$poa->set_parameters($r);
	}
	my $redir_url=$poa->{'GPoA_URL'};
	my $curl = $r->construct_url;
	my $poa_uri = APR::URI->parse($r->pool, $curl);	
	my $poa_sch=$poa_uri->scheme();
	my $poa_ref=$poa_uri->unparse();
	my $poa_server=$r->get_server_name;
	my $tsrv=$r->get_server_port;
	if ($tsrv ne "80" && $poa_sch eq "http") { $poa_server.=":$tsrv"; }
	if ($tsrv ne "443" && $poa_sch eq "https") { $poa_server.=":$tsrv"; }	
	my $qst=$r->args;
	my $hli = undef;
	my $opo = undef;
	my $opu = undef;
	for my $par (split /&/,$qst) {
		my ($pn,$pv)=split /=/,$par;
		if ($pn eq "PAPIHLI") { $hli = $pv; }
		elsif ($pn eq "PAPIOPOA") { $opo = $pv; }
		elsif ($pn eq "PAPIPOAURL") { $opu = $pv; }
		elsif ($pn eq "URL") { $opu = $pv; }
	}
	if (!defined $opo) { $opo = $opu; }	
	# Kan: change to UUID because too many connections with the same key
	# my $db_key=time.$$;
	$ug = new Data::UUID;
	my $db_key=$ug->create_str();
	$db_key =~ s/-//g;
	$poa->save_request($db_key);
	$poa_ref =~ s/\?.*$//g;
# The GPoA is a reference to connect to the adequate AS: Go to the WAYF URL
	if ($redir_url =~ /^wayf:/) {
		my $argstr.="PAPIPOAREF=".uri_escape($db_key,'\W')."&PAPIPOAURL=".uri_escape("$poa_ref",'\W');
		my ($wi, $wayfurl) = split /:/,$redir_url,2;
		my $asn=keys %{$poa->{ases}};
		if ($wayfurl =~ /^http/) {
			if ($wayfurl =~ /\?/) { $wayfurl.="&".$argstr; }
			else { $wayfurl.="?".$argstr; }
			if (defined $opo) { $wayfurl.="&PAPIOPOA=".$opo; }
			if (defined $hli) { $wayfurl.="&PAPIHLI=".$hli; }		
			elsif ($asn == 1) {
				my @aux = keys %{$poa->{ases}};
				$hli=pop @aux;
				$wayfurl.="&PAPIHLI=".uri_escape("$hli",'\W');
			}			
			$poa->debug("PAPI#RedirectGPoA# Redirecting to external WAYF: $wayfurl") if ($poa->{'Debug'});
			return PAPI::Cookie_Handler::redirect_URL($r,$wayfurl);
		}
		else {
			$argstr.="&NEXTURL=".uri_escape($poa_sch."://".$poa_server.$poa->{Loc}.$poa->{Auth_Location}."/PAPIASRedirector",'\W');
			$argstr.="&NOOFAS=".uri_escape($asn);
			$asn=0;
			for my $as (keys %{$poa->{ases}}) {
				$argstr.="&ASID".$asn."=".uri_escape($as,'\W')."&ASDESC".$asn."=".uri_escape($poa->{ases}{$as}->{desc},'\W')."&ASURL".$asn."=".uri_escape($poa->{ases}{$as}->{url},'\W');
				$asn++;
			}
			if (defined $opo) { $argstr.="&PAPIOPOA=".$opo; }
			if (defined $hli) { $argstr.="&PAPIHLI=".$hli; }
			if ($wayfurl =~ /built-in/i || ! -r $wayfurl) {
                $r->args($argstr);
                $poa->debug("PAPI#RedirectGPoA# Redirecting to built-in WAYF: $wayfurl") if ($poa->{'Debug'});
				return builtinWayf($r,$poa);
			}
			else {
				$poa->debug("PAPI#RedirectGPoA# Redirecting to internal WAYF: $wayfurl") if ($poa->{'Debug'}); 
				my $sr=$r->lookup_file($wayfurl);
				$sr->args($argstr);
				$sr->no_cache(1);
				return $sr->run();
			}
		}
	}
# Redirect to a normal GPoA
	else {
		my $argstr="ACTION=CHECK&DATA=$db_key&URL=".uri_escape("$poa_ref",'\W');
		if (defined $opo) { $argstr.="&PAPIOPOA=".$opo; }
		if (defined $hli) { $argstr.="&PAPIHLI=".$hli; }
		my $rurl = $redir_url;
		if ($redir_url =~ /\?/) { $rurl.="&".$argstr; }
		else { $rurl.="?".$argstr; }
		$poa->debug("PAPI#RedirectGPoA# Redirecting to: $rurl") if ($poa->{'Debug'}); 
		return PAPI::Cookie_Handler::redirect_URL($r,$rurl);
	}
}

sub builtinWayf
{
	my $r=shift;
	my $poa=shift;
	$r->no_cache(1);
	my %args = $poa->stringArgsToHash($r->args());
	my $action=uri_unescape($args{NEXTURL},'\W');	
	my $asn=$args{NOOFAS};
	my $poaref = $args{PAPIPOAREF};
	my $poaurl = $args{PAPIPOAURL};
	my $opoa = $args{PAPIOPOA};

# If there is just one AS, don't ask the user: just redirect
	if ($asn == 1) {
		my $id = $args{"ASID0"};
		my $redir=$action.'?PAPIPOAREF='.$poaref.'&PAPIPOAURL='.$poaurl."&ASID=$id";
		if (defined $opoa) { $redir.="&PAPIOPOA=".$opoa; }
		$poa->debug("REDIR: $redir") if ($poa->{'Debug'});
		return PAPI::Cookie_Handler::redirect_URL($r,$redir);
	}
    $r->content_type('text/html');
	print <<ENDOFPREAMBLE;
<html>
<head><title>PAPI Internal Location Service</title></head>
<body><form method="get" action="$action">
<h3>PAPI Internal Location Service</h3>
<p>Please select your Authentication Server from the following list and press the <b>Authenticate</b> button</p>
<input type=hidden name="PAPIPOAREF" value="$poaref">
<input type=hidden name="PAPIPOAURL" value="$poaurl">
<input type=hidden name="PAPIOPOA" value="$opoa">
<select name="ASID">
ENDOFPREAMBLE
	for (my $i=0; $i<$asn; $i++) {
		my $id=uri_escape($args{"ASID".$i},'\W');
		my $dsc=$args{"ASDESC".$i};
		$r->print("<option value=\"$id\">$dsc</option>\n");
	}
	print <<ENDOFPOSTAMBLE;
</select>&nbsp;&nbsp;&nbsp;<input type="submit" value="Authenticate">
</form></body>
</html>
ENDOFPOSTAMBLE
	return OK;
}

1;
 
__END__
